/*
    This file is part of Accountant.

    Accountant is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Accountant is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with Accountant.  If not, see <http://www.gnu.org/licenses/>.
 */
accountantApp.controller(
    "EntryController", function($scope, $http, $rootScope, $filter) {
    // Entry store and selection
    $scope.entries = [];
    $scope.categories = [];
    $scope.chartValues = [];
    $scope.pieChartValues = [];
    $scope.selectedItem = null;

    $scope.account = null;

    // Placeholder for saved value to cancel entry edition
    $scope.savedItem = null;

    $scope.begin = moment.utc().startOf('month');
    $scope.end = moment.utc().endOf('month');

    $scope.setExtremes = function(e) {
        begin = moment.utc(e.min);
        end = moment.utc(e.max);

        $scope.selectRange(begin, end);
    };

    $scope.selectRange = function(begin, end) {
        $scope.begin = begin;
        $scope.end = end;

        $scope.$emit("rangeSelectedEvent", {begin: begin, end: end});
    };

    // Configure pie chart for categories.
    $scope.categoriesChartConfig = {
        options: {
            chart: {
                type: 'pie',
                animation: {
                    duration: 500
                }
            },
            plotOptions: {
                pie: {
                    startAngle: -90
                }
            },
            tooltip: {
                valueDecimals: 2,
                valueSuffix: '€'
            }
        },
        yAxis: {
            title: {
                text: "Categories"
            }
        },
        title: {
            text: "Répartition dépenses/recettes"
        },
        series: [{
            name: "Value",
            data: [],
            innerSize: '33%',
            size: '60%',
            dataLabels: {
                formatter: function() {
                    return this.point.name;
                },
                distance: -40
            }
        }, {
            name: "Value",
            data: [],
            innerSize: '66%',
            size: '60%',
            dataLabels: {
                formatter: function() {
                    return this.point.name !== null && this.percentage >= 2.5 ? this.point.name : null;
                },
            }
        }]
    };

    // Configure chart for entries.
    $scope.soldChartConfig = {
        options: {
            chart: {
                zoomType: 'x'
            },
            rangeSelector: {
                buttons: [{
                    type: 'month',
                    count: 1,
                    text: "1m"
                }, {
                    type: "month",
                    count: 3,
                    text: "3m"
                }, {
                    type: "month",
                    count: 6,
                    text: "6m"
                }, {
                    type: "year",
                    count: 1,
                    text: "1y"
                }, {
                    type: "all",
                    text: "All"
                }],
                selected: 0,
            },
            navigator: {
                enabled: true
            },
            tooltip: {
                crosshairs: true,
                shared: true,
                valueDecimals: 2,
                valueSuffix: '€'
            }
        },
        series: [{
            type: "ohlc",
            name: "Sold",
            data: [],
            dataGrouping : {
                units : [[
                    'week', // unit name
                    [1] // allowed multiples
                ], [
                    'month',
                    [1, 2, 3, 4, 6]
                ]]
            }
        }],
        title: {
            text: "Sold evolution"
        },
        xAxis: {
            type: "datetime",
            dateTimeLabelFormats: {
                month: "%e. %b",
                year: "%Y"
            },
            minRange: 3600 * 1000 * 24 * 14, // 2 weeks
            events: {
                afterSetExtremes: $scope.setExtremes
            }
        },
        yAxis: {
            plotLines: [{
                color: "orange",
                width: 2,
                value: 0.0
            }, {
                color: "red",
                width: 2,
                value: 0.0
            }]
        },
        useHighStocks: true
    };

    // Load categories, mainly to populate the pie chart.
    $scope.loadCategories = function() {
        $http.get("/api/categories", {
            params: {
                account: $scope.account.id,
                begin: $scope.begin.format('YYYY-MM-DD'),
                end: $scope.end.format('YYYY-MM-DD')
            }
        }).success(function(data) {
            var expenses = [], revenues = [];
            var colors = Highcharts.getOptions().colors;

            var config = $scope.categoriesChartConfig;

            angular.forEach(angular.fromJson(data), function(category) {
                brightness = 0.2;

                expenses.push({
                    name: category.category,
                    y: -category.expenses,
                    color: Highcharts.Color(config.series[0].data[1].color).brighten(brightness).get()
                });

                revenues.push({
                    name: category.category,
                    y: category.revenues,
                    color: Highcharts.Color(config.series[0].data[0].color).brighten(brightness).get()
                });
            });

            // Note: expenses and revenues must be in the same order than in series[0].
            config.series[1].data = revenues.concat(expenses);

            $scope.categoriesChartConfig.loaded = true;

            $scope.loadSolds();
        });
    };

    $scope.loadSolds = function() {
        $scope.soldChartConfig.loaded = true;

        $http.get("/api/solds", {
            params: {
                account: $scope.account.id,
            }
        }).success(function(data) {
            var config = $scope.soldChartConfig;

            config.series[0].data = [];

            angular.forEach(data, function(entry) {
                config.series[0].data.push([
                    moment.utc(entry.operation_date).valueOf(),
                    entry.open, entry.high, entry.low, entry.close
                ]);
            });

            $scope.loadEntries();
        });
    };

    $rootScope.$on("accountSelectedEvent", function(event, args) {
        $scope.account = args.account;

        $scope.getAccountStatus();
    });

    $rootScope.$on("rangeSelectedEvent", function(event, args) {
        $scope.getAccountStatus();
    });

    $rootScope.$on("entriesLoadedEvent", function(event, args) {
        //$scope.loadCategories();
    });

    $scope.getAccountStatus = function() {
        $scope.categoriesChartConfig.loaded = true;

        $http.get("/api/accounts/" + $scope.account.id, {
            params: {
                begin: $scope.begin.format('YYYY-MM-DD'),
                end: $scope.end.format('YYYY-MM-DD')
            }
        }).success(function(status) {
            var colors = Highcharts.getOptions().colors;

            var config = $scope.categoriesChartConfig;

            // Update pie chart subtitle with Balance.
            config.subtitle = {
                text: "Balance: " + status.balance
            };

            config.series[0].data = [{
                name: "Revenues",
                y: status.revenues,
                color: colors[2]
            }, {
                name: "Expenses",
                y: -status.expenses,
                color: colors[3],
            }];

            $scope.soldChartConfig.yAxis.plotLines[1].value = status.authorized_overdraft;

            $scope.loadCategories();
        });
    };

    // Function to load entries from server for a specific account and month.
    $scope.loadEntries = function() {
        // Clean up selected entry.
        $scope.selectedItem = null;
        $scope.savedItem = null;

        if(!$scope.entriesLoading) {
            $scope.entriesLoading = true;

            $http.get("/api/entries", {
                params: {
                    account: $scope.account.id,
                    begin: $scope.begin.format('YYYY-MM-DD'),
                    end: $scope.end.format('YYYY-MM-DD')
                }
            }).success(function(data) {
                var entries = [{
                id: null,
                pointed: false,
                operation_date: null,
                label: null,
                value: null,
                sold: null,
                pointedsold: null,
                category: null,
                account_id: null,
                state: 'new',
                canceled: false,
                scheduled_operation_id: null
                }];

                if(data) {
                    entries = entries.concat(angular.fromJson(data));
                }

                $scope.entries = entries;

                $scope.entriesLoading = false;

                $scope.$emit("entriesLoadedEvent", {entries: entries});
            });
        }
    };

    // Returns the CSS class for a pointed entry.
    $scope.pointedEntryClass = function(entry) {
        if(entry.pointed) {
            return "active";
        }
    };

    // Returns the CSS class for an entry row.
    $scope.entryRowClass = function(entry) {
        if($scope.isSaved(entry)) {
            cssclass="";
        } else {
            cssclass="italic";
        }

        if(entry.canceled) {
            cssclass += " stroke";
        }

        if(entry.sold < $scope.account.authorized_overdraft) {
            cssclass += " danger";
        } else if (entry.sold < 0) {
            cssclass += " warning";
        }

        return cssclass;
    };

    // Returns the CSS class for an entry sold.
    $scope.entryValueClass = function(sold) {
        if(sold && sold < $scope.account.authorized_overdraft) {
            return 'text-danger';
        } else if (sold && sold < 0) {
            return 'text-warning';
        }
    };

    // Starts editing an entry
    $scope.editEntry = function(entry) {
        // Cancel previous editing.
        if($scope.selectedItem) {
            $scope.cancelEditEntry($scope.selectedItem);
        }

        // Save current entry values.
        $scope.savedItem = angular.copy(entry);
        $scope.selectedItem = entry;

        // Enter edit state.
        entry.state='edit';
    };

    // Returns true if the entry is a new one.
    $scope.isNew = function(entry) {
      return !$scope.isSaved(entry) && (entry.state === 'edit' || entry.state === 'new');
    };

    // Returns true if the entry is in editing state.
    $scope.isEditing = function(entry) {
      return entry.state === 'edit' || entry.state === 'new';
    };

    // Returns true if the entry is in displaying state.
    $scope.isDisplaying = function(entry) {
      return !entry.state || entry.state === 'display';
    };

    // Returns true if the entry is a scheduled one.
    $scope.isSaved = function(entry) {
        return entry.id !== null;
    };

    $scope.iconSaveClass = function(entry) {
      if(!$scope.isSaved(entry)) {
        return "fa fa-plus";
      } else if ($scope.isEditing(entry)) {
        return "fa fa-floppy-o";
      }
    };

    $scope.iconCancelClass = function(entry) {
      if($scope.isNew(entry)) {
        return "fa fa-times";
      } else if ($scope.isEditing(entry)) {
        return "fa fa-ban";
      }
    };

    // Cancel current editing entry or clears field if a new one.
    $scope.cancelEditEntry = function(entry) {
      if ($scope.savedItem === null) {
        // We are cancelling the new entry, just reset it.
        entry.id = null; // id should not change, but just in case...
        entry.pointed = false;
        entry.operation_date = null;
        entry.label = null;
        entry.value = null;
        entry.account_id = $scope.account.id; // account_id should not change, but just in case...
        entry.category = null;
        entry.canceled = false;
        entry.scheduled_operation_id = null;

        // Reset state to new.
        entry.state = 'new';
      } else {
        // Reset selected item fields to saved item ones.
        entry.id = $scope.savedItem.id; // id should not change, but just in case...
        entry.pointed = $scope.savedItem.pointed;
        entry.operation_date = $scope.savedItem.operation_date;
        entry.label = $scope.savedItem.label;
        entry.value = $scope.savedItem.value;
        entry.account_id = $scope.savedItem.account_id; // account_id should not change, but just in case...
        entry.category = $scope.savedItem.category;
        entry.canceled = $scope.savedItem.canceled;
        entry.scheduled_operation_id = $scope.savedItem.scheduled_operation_id;

        // Reset saved and selected items to null.
        $scope.savedItem = null;
        $scope.selectedItem = null;

        // Enter display state.
        entry.state = 'display';
      }
    };

    // Points an entry.
    $scope.pointEntry = function(entry) {
      if(entry.pointed) {
        entry.pointed = false;
      } else {
        entry.pointed = true;
      }

      // Save the entry if not new.
      if(!$scope.isNew(entry)) {
        $scope.saveEntry(entry);
      }
    };

    // Saves an entry.
    $scope.saveEntry = function(entry) {
      // Affects the account ID if not (should never happen)
      if(!entry.account_id) {
          entry.account_id = $scope.account.id;
      }

      // Prepare the Ajax call to save the entry.
      var type;
      var url = "/api/entries";

      if($scope.isSaved(entry)) {
        url += "/" + entry.id;
      }

      // Ajax call to save an entry
      $http.put(url, angular.toJson(entry)).success(function(data) {
        $.pnotify({
            type: "success",
            title: "Save",
            text: data
        });

        // $scope.savedItem = null;

        // Send the "entry saved" event.
        $scope.$emit("entrySavedEvent", entry);
      });
    };

    $scope.removeEntry = function(entry) {
        $scope.removingEntry = entry;
        $("#remove_entry").modal({
            keyboard: false,
        });
    };

    $scope.hideRemoveEntryPopup = function() {
        $scope.removingEntry = null;
        $("#remove_entry").modal("hide");
    };

    // Removes an entry.
    $scope.confirmRemoveEntry = function() {
      // Cancel current editing.
      if ($scope.removingEntry) {
        $http.delete("/api/entries/" + $scope.removingEntry.id).success(function (result) {
          $.pnotify({
              type: "success",
              title: "Delete",
              text: result
          });

          // Send the "entry removed" event.
          $scope.$emit("entryRemovedEvent", $scope.removingEntry);

          $scope.hideRemoveEntryPopup();
        });
      }
    };

    $scope.closeModal = function(modalScope) {
      // Close the modal dialog
      if(modalScope && modalScope.dismiss) {
        modalScope.dismiss();
      }
    };
});
